
using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Design;
using System.Reflection;
using System.Windows.Forms;

using Syncfusion.Styles;
using Syncfusion.ComponentModel;

using Syncfusion.Windows.Forms.Grid;

namespace CustomColumnExample
{
	/// <summary>
	/// An editor for a collection of <see cref="GridBoundColumn"/>s. This is
	/// a specialized version which creates 
	/// </summary>
	public class ColumnsCollectionEditor : GridBoundColumnsCollectionEditor
	{
        /// <summary>
        /// Default constructor.
        /// </summary>
		public ColumnsCollectionEditor(Type type)
            : base( typeof(CustomGridColumn) )
		{
		}

		       
		object _EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
		{
			return base.EditValue(context, provider, value);
		}

		/// <override/>
		public override /*UITypeEditor*/ object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
		{
			if (context == null || context.Instance == null || context.PropertyDescriptor == null
				|| context.PropertyDescriptor.IsReadOnly)
				return _EditValue(context, provider, value);

			ColumnsCollectionEditor ce = new ColumnsCollectionEditor(this.CollectionType);
			WindowsFormsEditorServiceContainer esc = new WindowsFormsEditorServiceContainer(provider);
			PropertyDescriptor pd = context.PropertyDescriptor;
			object instance = context.Instance;
			TypeDescriptorContext tdc = new TypeDescriptorContext(instance, pd);
			tdc.ServiceProvider = esc;

			IDesignerHost designerHost = (IDesignerHost) provider.GetService(typeof(IDesignerHost));
							
			IComponentChangeService changeService = null;
			if (provider != null)
				changeService = provider.GetService(typeof(IComponentChangeService)) as IComponentChangeService;
							
			DesignerTransaction designerTransaction = null;
			if (designerHost != null)
				designerTransaction = designerHost.CreateTransaction("Change " + pd.Name);

			object oldValue = pd.GetValue(instance);
			object editValue = ((ICloneable) oldValue).Clone();
			object resultValue = ce._EditValue(tdc, esc, editValue);
			// resultValue is normally the same as editValue
			// Object.ReferenceEquals(resultValue, editValue) should be true

			if (esc.DialogResult == DialogResult.OK)
			{
				try
				{
					//changeService.OnComponentChanging(instance, pd);
					pd.SetValue(instance, editValue);
					//changeService.OnComponentChanged(instance, pd, oldValue, editValue);
				}
				finally
				{
					if (designerTransaction != null)
						designerTransaction.Commit();
				}

				Control control = instance as Control;
				if (control != null)
					control.Invalidate();
			}
			else
			{
				if (designerTransaction != null)
					designerTransaction.Cancel();
				return oldValue;
			}

			return resultValue;
		}

	}
}
