
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Design;

using Syncfusion.Windows.Forms.Grid;

namespace CustomColumnExample
{
	public class SyncFusionBoundGrid : GridDataBoundGrid
	{
        /// <summary>
        /// Default constructor.
        /// </summary>
		public SyncFusionBoundGrid() : base()
        {
			GridModelDataBinder.BoundColumnsCollectionType = typeof(CustomGridColumnsCollection);
		 }

		private void model_QueryColWidth(object sender, GridRowColSizeEventArgs e)
		{
			if(this.CustomGridColumns != null)
			{
				int field = this.Binder.ColIndexToField(e.Index);
				if(field > -1 && field < this.CustomGridColumns.Count 
					&& this.CustomGridColumns[field].Width > 0)
				{
					e.Size = this.CustomGridColumns[field].Width;
					e.Handled = true;
					
				}
			}
		}

        /// <summary>
        /// A collection of grid bound columns. This property is over-ridden so
        /// that the type used to define a grid bound column is a specialized
        /// version.
        /// </summary>
        [ Browsable(false), DesignerSerializationVisibilityAttribute( DesignerSerializationVisibility.Hidden ) ]
        public override GridBoundColumnsCollection GridBoundColumns
        {
            get
            {
                return base.GridBoundColumns;
            }
            set
            {

                base.GridBoundColumns = value;
            }
        }

		/// <summary>
		/// A collection of grid bound columns. This property is over-ridden so
		/// that the type used to define a grid bound column is a specialized
		/// version.
		/// </summary>
		[ DesignerSerializationVisibilityAttribute( DesignerSerializationVisibility.Content ) ]
		[ LocalizableAttribute(true) ]
		[ Description( "Manages the columns to be displayed in the GridDataBoundGrid." ) ] 
		[ Category( "Data" ) ]
		[ RefreshProperties( RefreshProperties.All ) ]
		[ Editor( typeof(GridBoundColumnsCollectionEditor), typeof(UITypeEditor) ) ]
		public CustomGridColumnsCollection CustomGridColumns
		{
			get
			{
				return base.GridBoundColumns as CustomGridColumnsCollection;
			}
			set
			{

				base.GridBoundColumns = value;
			}
		}

		
		private bool onceOnly = true;
		protected override void OnPaint(System.Windows.Forms.PaintEventArgs pe)
		{
			if(onceOnly)
			{
				this.Model.QueryColWidth += new GridRowColSizeEventHandler(model_QueryColWidth);
				onceOnly = false;
				this.Model.ResetVolatileData();
				this.UpdateScrollBars();
			}

			base.OnPaint(pe);
		}
	}
}
